//
//  AMFloatingImage.m
//  AMGradientPanel
//
//  Created by Andreas on 13.01.10.
//  Copyright 2010 Andreas Mayer. All rights reserved.
//

#import "AMFloatingImage.h"
#import "AMImageView.h"


@interface AMFloatingImage ()
	@property (retain, nonatomic) NSTimer *trackingTimer;
	- (void)trackCursorPosition;
@end


@implementation AMFloatingImage

@synthesize hotSpot;
@synthesize followCursor;
@synthesize trackingTimer;

- (id)initWithImage:(NSImage *)theImage hotSpot:(NSPoint)point
{
	NSRect rect = NSZeroRect;
	rect.size = [theImage size];
	if ((self = [super initWithContentRect:rect styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO])) {
		[self setHotSpot:point];
		[self setFollowCursor:NO];
		
		[self setOpaque:NO];
		[self setBackgroundColor: [NSColor clearColor]];
		[self setCollectionBehavior:NSWindowCollectionBehaviorCanJoinAllSpaces];
		[self setLevel:kCGDraggingWindowLevel];
		[self setIgnoresMouseEvents:YES];
		[self setReleasedWhenClosed:NO];
		
		NSView *imageView = [[AMImageView alloc] initWithFrame:rect image:theImage];
		[self setContentView:imageView];
		[imageView release];
	}
	return self;
}

- (void)dealloc
{
	[trackingTimer invalidate];
	[trackingTimer release];
	[super dealloc];
}

- (void)moveToPoint:(NSPoint)point
{
	NSPoint newOrigin = point;
	newOrigin.x -= hotSpot.x;
	newOrigin.y -= hotSpot.y;
	[self setFrameOrigin:newOrigin];
	[self orderFrontRegardless];
}

- (void)setFollowCursor:(BOOL)flag
{
	if (flag != followCursor) {
		followCursor = flag;
		if (followCursor) {
			// follow
			NSTimer *timer = [NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(trackCursorPosition) userInfo:NULL repeats:YES];
			[self setTrackingTimer:timer];
			[self trackCursorPosition];
		} else {
			// un-follow
			[trackingTimer invalidate];
			[self setTrackingTimer:nil];
		}

	}
}

- (void)trackCursorPosition
{
	NSPoint cursorPoint = [NSEvent mouseLocation];
	[self moveToPoint:cursorPoint];
}


@end
